# copyright 2022 Medicines Discovery Catapult
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from unittest import mock
from klein_config import EnvironmentAwareConfig
from src.klein_mongo import MongoConnection, ConfigError
import pytest

config = EnvironmentAwareConfig({
    "mongo": {
        "host": ["mongo.domain.com"],
        "username": "me",
        "password": "secret",
    }
})


class TestPyMongoClient:

    def test_error(self):
        with pytest.raises(ConfigError) as e:
            MongoConnection(
                EnvironmentAwareConfig({
                    "mongo": {
                        "host": 23424,
                        "username": "me",
                        "password": "hello"
                    }
                }))
            assert str(e) == "mongo hosts must be string or list of strings"

    def test_connection(self):
        connection = MongoConnection(config)
        assert isinstance(connection.host, list)
        assert len(connection.host) == 2
        assert connection.host[0] == "mongo.domain.com"
        assert connection.host[1] == 27017

    @mock.patch.dict(os.environ, {"MONGO_HOST": "mongo0.domain.com,mongo1.domain.com,mongo2.domain.com"})
    def test_multiple_hosts(self):
        connection = MongoConnection(config)
        assert isinstance(connection.host, list)
        assert len(connection.host) == 2
        assert connection.host[0] == "mongo0.domain.com,mongo1.domain.com,mongo2.domain.com"
        assert connection.host[1] == 27017

    @mock.patch.dict(os.environ, {"MONGO_SRV": "true"})
    @mock.patch("pymongo.srv_resolver._SrvResolver.get_hosts",
                return_value=["mongo0.domain.com", "mongo1.domain.com", "mongo2.domain.com"])
    @mock.patch("pymongo.srv_resolver._SrvResolver.get_options", return_value="replicaSet=mongo")
    def test_srv(self, mock_hosts, mock_config):
        connection = MongoConnection(config)
        assert isinstance(connection.host, list)
        assert len(connection.host) == 1
        assert connection.host[0] == "mongodb+srv://mongo.domain.com"
        assert connection.params["username"] == "me"
        assert connection.params["password"] == "secret"
        assert connection.params["uuidRepresentation"] == "standard"



# copyright 2022 Medicines Discovery Catapult
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from setuptools import setup, find_packages
from pathlib import Path
this_directory = Path(__file__).parent
readme_description = (this_directory / "README.md").read_text()


def readme():
    with open('README.md') as f:
        return f.read()


__version__ = ''
exec(open("./src/version.py").read())
if __version__ == '':
    raise RuntimeError("unable to find application version")

setup(name='klein_mongo',
      version=__version__,
      description='MongoDB integration',
      long_description=readme_description,
      long_description_content_type='text/markdown',
      url='https://github.com/mdcatapult/py-mongo',
      author='Medicines Discovery Catapult',
      author_email='SoftwareEngineering@md.catapult.org.uk',
      license='Apache V2',
      packages=find_packages('src'),
      package_dir={'':'src'},
      install_requires=[
          'klein_config>=3.0, <4',
          'pymongo[srv]>=3.11, <4',
      ],
      zip_safe=True,
      include_package_data=True)



# copyright 2022 Medicines Discovery Catapult
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pymongo import MongoClient as _MongoClient


def lazy_property(fn):
    '''Decorator that makes a property lazy-evaluated.
    '''
    attr_name = '_lazy_' + fn.__name__

    @property
    def _lazy_property(self):
        if not hasattr(self, attr_name):
            setattr(self, attr_name, fn(self))
        return getattr(self, attr_name)
    return _lazy_property


class ConfigError(Exception):
    pass


class MongoConnection:

    def __init__(self, config):
        self.params = {}

        self.replicaSet = None

        if config.has('mongo.username'):
            self.params["username"] = config.get('mongo.username')

        if config.has('mongo.password'):
            self.params["password"] = config.get('mongo.password')

        if config.has('mongo.authSource'):
            self.params["authSource"] = config.get('mongo.authSource')

        elif config.has('mongo.database') and "username" in self.params:
            self.params["authSource"] = config.get('mongo.database')

        if config.has('mongo.authMechanism') and "password" in self.params:
            self.params["authMechanism"] = config.get('mongo.authMechanism')

        host = config.get('mongo.host')
        srv = config.get('mongo.srv', False)
        use_srv = srv is True or (isinstance(srv, str) and srv.lower() in ['true', '1', 'yes'])
        if isinstance(host, str):
            if use_srv:
                self.host = [f"mongodb+srv://{host}"]
            else:
                self.host = [host, int(config.get('mongo.port', 27017))]
        elif isinstance(host, list):
            if use_srv:
                self.host = [f"mongodb+srv://{host[0]}"]
            else:
                self.host = [",".join(config.get("mongo.host")), int(config.get('mongo.port', 27017))]
        else:
            raise ConfigError("mongo hosts must be string or list of strings")

        if config.has('mongo.replicaSet'):
            self.replicaSet = config.get('mongo.replicaSet')

        self.params["readPreference"] = config.get('mongo.readPreference', 'secondaryPreferred')

        # Use cross language compatible UUID encoding as default. Can be overridden with
        # 'pythonLegacy', 'javaLegacy' or 'csharpLegacy' if required.
        # ref: https://api.mongodb.com/python/current/api/pymongo/mongo_client.html?
        self.params['uuidRepresentation'] = config.get('mongo.uuidRepresentation', 'standard')

    @lazy_property
    def client(self):
        c = _MongoClient(*self.host, replicaset=self.replicaSet, **self.params)
        return c


def get_client(config):
    return MongoConnection(config).client
